/* Note this is a generated file.
Before editing this file directly consider editing the script
generateFilterCode.pl
which generates the idl file and much of the required javascript
code for an XPCOM component
*/

/***********************************************************
constants
***********************************************************/

// reference to the interface defined in cmguiICannyEdgeDetectionFilter.idl
const cmguiICannyEdgeDetectionFilter = Components.interfaces.cmguiICannyEdgeDetectionFilter;

// reference to the required base interface that all components must support
const nsISupports = Components.interfaces.nsISupports;

// references to other required interfaces
// const nsIDOMWindow = Components.interfaces.nsIDOMWindow;

// F33C4D5E-F6BE-11DB-9265-7BB155D89593 uniquely identifying our component
const CLASS_ID = Components.ID("{F33C4D5E-F6BE-11DB-9265-7BB155D89593}");

// description
const CLASS_NAME = "Cmgui canny edge detection filter XPCOM Component";

// textual unique identifier
const CONTRACT_ID = "@bioeng.auckland.ac.nz/cannyedgedetectionfilter;1";

/***********************************************************
class definition
***********************************************************/

//class constructor
function CannyEdgeDetectionFilter() {
};


// class definition
CannyEdgeDetectionFilter.prototype = {

  // members
  parent: null,
  commandData: null,
  variance: 1,
  maximumError: 0.01,
  upperThreshold: 0,
  lowerThreshold: 0,
  inputFieldName: null,
  inputField: null,
  outputFieldName: null,
  outputField: null,
  callback: null,
  previewOutputFieldName: null,
  previewOutputField: null,
  gElementName: null,
  sceneViewer: null,
  previewResolution: 128,  // should be a power of 2

// define the function methods we want to expose in our interface

  init: function(parentWindow, commandData, inputFieldName, outputFieldName, callback) {
    
    this.parent = parentWindow;
    this.commandData = commandData;         // command data object for accessing cmgui commands    
    this.inputFieldName = inputFieldName;	
    this.outputFieldName = outputFieldName;
    this.callback = callback;
		
    // find the input field, note it is assumed to exist
    this.inputField = commandData.findFieldByName(inputFieldName);
    
    // set some defaults which will be overwritten if the field exists
    this.variance = 1;
    this.maximumError = 0.01;
    this.upperThreshold = 0;
    this.lowerThreshold = 0;
    
    // check if output field exists
    if( commandData.isFieldDefined(outputFieldName) ){
      
      // field exists so fetch it
      this.outputField = commandData.findFieldByName(outputFieldName);
      
      // create some objects which will be populated by the current field
      // properties if the field is the right type
      
      var returnInputField = new Object();
      var variance = new Object();
      var maximumError = new Object();
      var upperThreshold = new Object();
      var lowerThreshold = new Object();
      
      try {
	
	this.outputField.getTypeCannyEdgeDetectionImageFilter(returnInputField,variance, maximumError, upperThreshold, lowerThreshold);
	
	// when fetched the parameters are assigned to the value property 
	this.variance = variance.value;
	this.maximumError = maximumError.value;
	this.upperThreshold = upperThreshold.value;
	this.lowerThreshold = lowerThreshold.value;
				
      } catch(ex) {
	
	this.parent.alert("Could not fetch parameters, probably because " + 
			  "the output field exists but is of the wrong type " + ex);				
	this.outputField.setTypeCannyEdgeDetectionImageFilter(this.inputField,this.variance, this.maximumError, this.upperThreshold, this.lowerThreshold);
	
      }
      
    } else {
      
      // field does not exist so create it.
      
      this.outputField = commandData.createField(outputFieldName);
      this.outputField.setTypeCannyEdgeDetectionImageFilter(this.inputField,this.variance, this.maximumError, this.upperThreshold, this.lowerThreshold);
      
    }
    
  },
  
  
  setup_preview_field: function() {
    
    
    // first resample the input field
    // will eventually use createField and setType to do this
    //		this.resampledInputFieldName = "resampled_" + this.inputFieldName;
    //		this.commandData.executeCommand("gfx def field " + this.resampledInputFieldName +
    //   	 " image_resample field " + this.inputFieldName + " sizes 128 128;");
    //		this.resampledInputField = this.commandData.findFieldByName(this.resampledInputFieldName);
    
    // Set up previewOutputField
    this.previewOutputFieldName = "preview_" + this.outputFieldName;
    //		this.previewOutputField = this.commandData.createField(this.previewOutputFieldName);
    
    // set the type with the current options
    
    this.commandData.executeCommand("gfx def field " + this.previewOutputFieldName +
				    " image_resample field " + this.outputFieldName + 
				    " sizes " + this.previewResolution + " " + this.previewResolution + ";");
    
  },
  

  resample_preview: function() {
    
    this.commandData.executeCommand("gfx def field " + this.previewOutputFieldName +
				    " image_resample field " + this.outputFieldName + 
				    " sizes " + this.previewResolution + " " + this.previewResolution + ";");
    
  },


  // call this function from the dialog to take care of updating the filter and running the callback
  apply_filter: function() {
    
    this.outputField.setTypeCannyEdgeDetectionImageFilter(this.inputField,this.variance, this.maximumError, this.upperThreshold, this.lowerThreshold);			
    this.callback.run();
    
  },
  
  // should this return the name of the material?  change this to update_preview_field
  preview_filter: function() {
    
    // update the preview output field options
    this.outputField.setTypeCannyEdgeDetectionImageFilter(this.inputField,this.variance, this.maximumError, this.upperThreshold, this.lowerThreshold);			
    
  },

  
  create_dialog: function() {
    
    var parent = this.parent;
    var params={cannyEdgeDetectionFilter:this};
    
    this.setup_preview_field();
    
    try {
      
      dialogReference = parent.openDialog("chrome://zinc/content/filters/cannyEdgeDetectionFilterDialog.xul",
					  "canny-edge-detection-filter-dialog",
					  "chrome",params);
      
      // store reference to dialog and associated field name
      parent.cannyEdgeDetectionFilterDialogReference = dialogReference;
      parent.cannyEdgeDetectionFilterDialogReferenceOutputField = this.outputFieldName;
      
    }catch(ex) {
      parent.alert("Error creating dialog to edit canny edge detection filter: " + ex);
    }
    
  },
  
  
  open_dialog: function(gElementName, sceneViewer) {
    
    this.gElementName = gElementName;
    this.sceneViewer = sceneViewer;
    
    // window parent
    var parent = this.parent;
    var dialogReference = parent.cannyEdgeDetectionFilterDialogReference;
    
    // determine if a dialog already exists
    if (dialogReference == null || dialogReference.closed) {
      
      // no dialog so create one
      this.create_dialog();
      
    } else { 
      
      // dialog exists, check if it is relevant to this field.				
      var oldOutputFieldName = parent.cannyEdgeDetectionFilterDialogReferenceOutputField;
      
      if (oldOutputFieldName == this.outputFieldName) {
	
	// correct dialog so bring it to the front
	dialogReference.focus();
	
      } else {
	
	// not correct so close dialog and open a new one
	dialogReference.close();
	this.create_dialog();
	
      }
    }
    
  },
  
    

  // the QueryInterface function throws an exception
  // if the cmguiIGeneralThresholdFilter interface is not implemented
  // otherwise it returns a reference to the GeneralThresholdFilter object
  QueryInterface: function(aIID)
  {
    if (!aIID.equals(cmguiICannyEdgeDetectionFilter) &&    
        !aIID.equals(nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  }
};


/***********************************************************
class factory

This object is a member of the global-scope Components.classes.
It is keyed off of the contract ID. Eg:

myCannyEdgeDetectionFilter = Components.classes["@bioeng.auckland.ac.nz/helloworld;1"].
                          createInstance(Components.interfaces.cmguiICannyEdgeDetectionFilter);

***********************************************************/
var CannyEdgeDetectionFilterFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new CannyEdgeDetectionFilter()).QueryInterface(aIID);
  }
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/
var CannyEdgeDetectionFilterModule = {
  _firstTime: true,
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, 
        CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.
        QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);        
  },
  
  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return CannyEdgeDetectionFilterFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aCompMgr) { return true; }
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/
function NSGetModule(aCompMgr, aFileSpec) { return CannyEdgeDetectionFilterModule; }
